==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/SAPDB/Trace/Trace_Manager.cpp <==
/****************************************************************************

  module      : Trace_Manager.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : 

  last changed: 2000-11-27  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "heo51.h"  // RTE   : VgetUKTId
#include "heo55k.h" // RTE   : Vbegexcl/Vendexcl 
#include "heo56k.h" // RTE   : Vsuspend/Vresume
#include "heo57.h"  // RTE   : Vmalloc/Vmfree
#include "hgg01.h"  // PASCAL: Configuration
#include "hgg08.h"  // PASCAL: RegionIds
#include <new.h>    // SYSTEM: required for placement new

#include "SAPDB/Trace/Trace_Manager.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

Trace_Manager*  Trace_Manager::m_Instance = NULL;

/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

class Trace_Lock
{
public:
    
    Trace_Lock( tsp00_TaskId TaskId );

    ~Trace_Lock();

private:

    tsp00_TaskId    m_Owner;
    tsp00_RegionId  m_RegionId;
};

/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/


/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/* Singleton interface                                                       */
/*---------------------------------------------------------------------------*/

bool
Trace_Manager::CreateInstance( tsp00_TaskId TaskId )
{
    if ( NULL == m_Instance )
    {
        tsp00_Bool  bOkay   = FALSE;
        tsp00_Int4  NumUKTs = g01NumberOfUKTs();
        void        *pObj   = NULL;

        vmalloc( sizeof( Trace_Manager ), &pObj, &bOkay );

        if (TRUE != bOkay) return( false );

        m_Instance = REINTERPRET_CAST( Trace_Manager*, new( pObj ) Trace_Manager( NumUKTs ));

        if (! m_Instance->Initialize( TaskId )) return( false );
    }
    return( true );
};

/*---------------------------------------------------------------------------*/

Trace_Manager::Trace_Manager( tsp00_Int4    NumUKTs )
:
m_TraceWriterRegistered( false ),
m_NumUKTs( NumUKTs ),
m_UserWaitsForFlush( cgg_nil_pid ),
m_TraceWriterWaitsForWork ( cgg_nil_pid ),
m_SeqNoGenerator( TRACE_FIRST_SEQNO )
{
};

/*---------------------------------------------------------------------------*/

bool
Trace_Manager::Initialize( tsp00_TaskId TaskId )
{

    
    m_DevSpace.Initialize( m_DumpFile );
    
    if (! m_DevSpace.Open( TaskId )) return( false );

    if (! m_DevSpace.WriteInfoPage( TaskId )) return( false );

    return( m_SlotPool.Initialize( m_NumUKTs ) );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::Flush( tsp00_TaskId TaskId )
{
    Trace_Lock              Lock( TaskId );
    Trace_SlotPoolIterator  SlotIter( m_SlotPool );
    
    
    if ( ! m_TraceWriterRegistered ) return;
    
    // Flush memory trace into knltrace file
    for( SlotIter.Begin(); !SlotIter.End(); ++SlotIter )
    {
        ( *SlotIter ).Flush( TaskId, m_DevSpace);
    }

    // Resume user task waiting for trace flush
    if ( cgg_nil_pid != m_UserWaitsForFlush )
    {
        vresume( m_UserWaitsForFlush, cgg_suspend_lvl_0 );
        m_UserWaitsForFlush = cgg_nil_pid;
    }
};

/*---------------------------------------------------------------------------*/

tsp00_VFilename 
Trace_Manager::GetDumpFile()
{
    return( m_DumpFile );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::Insert(
                      tgg00_TransContext    &Trans,
                      Trace_IBaseEntry      &Entry )
{
    if (! WriteEntryPossible( Trans, Entry.GetSize() )) return;

    // This method is not able to materialize trace entries greater than 132 bytes
    if ( Entry.MaximumSize() < Entry.GetSize() ) return;
    
    // Get the RTE internal number of the UKT in which the current task is running.
    tsp00_Int4  OwnUKTId = 0;
    
    vgetuktid( OwnUKTId );
    
    m_SlotPool.Get( OwnUKTId ).Insert( Trans, OwnUKTId, Entry, m_SeqNoGenerator );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::InsertLarge( 
                                tgg00_TransContext  &Trans,
                                Trace_ILargeEntry   &Entry )
{
    if (! WriteEntryPossible( Trans, Entry.GetSize() )) return;

    // Get the RTE internal number of the UKT in which the current task is running.
    tsp00_Int4  OwnUKTId   = 0;
    
    vgetuktid( OwnUKTId );
    
    m_SlotPool.Get( OwnUKTId ).InsertLarge( Trans, OwnUKTId, Entry, m_SeqNoGenerator );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::PrepareAndWaitForFlush( 
                                      tsp00_TaskId TaskId,
                                      tsp00_Bool   bReInitialization )
{
    {
        Trace_Lock              Lock( TaskId );
        Trace_SlotPoolIterator  SlotIter( m_SlotPool );

        
        if ( ! m_TraceWriterRegistered ) return;
        
        if (TRUE == bReInitialization)
        {
            // Reset the sequence number generater for the trace entries
            m_SeqNoGenerator.SetValue( TRACE_FIRST_SEQNO );

            for( SlotIter.Begin(); !SlotIter.End(); ++SlotIter )
            {
                ( *SlotIter ).ReInitialization();
            }
        };

        // Resume the trace writer to flush UKT specific trace slots
        if ( cgg_nil_pid != m_TraceWriterWaitsForWork )
        {
            vresume( m_TraceWriterWaitsForWork, cgg_suspend_lvl_0 );
            m_TraceWriterWaitsForWork = cgg_nil_pid;
        }
        
        // Store user taskid to enable wake up after the trace writer has
        // flushed the complete trace.
        m_UserWaitsForFlush = TaskId; 
    }
    vsuspend (TaskId, cgg_suspend_lvl_0, 51);
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::TraceWriterRegistration( tsp00_TaskId TaskId )
{
    Trace_Lock Lock( TaskId );
    
    m_TraceWriterRegistered = true;
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::WaitForWork( tsp00_TaskId TaskId )
{
    {
        Trace_Lock Lock( TaskId );
        
        m_TraceWriterWaitsForWork = TaskId;
    }
    vsuspend (TaskId, cgg_suspend_lvl_0, 49);
};

/*---------------------------------------------------------------------------*/

bool
Trace_Manager::WriteEntryPossible( 
                                  tgg00_TransContext   &Trans,
                                  tsp00_Int4           EntrySize )
{
    if ( ! m_TraceWriterRegistered ) return( false );
    
    // if session specific trace is enabled than only the specified session 
    // has the right to write into the kernel trace.
    if ( 
        (! g01vtrace.vtrSession_gg00.gg90IsNil() ) && 
        ( g01vtrace.vtrSession_gg00 != Trans.trSessionId_gg00 ) 
        )
        return( false );
    
    if  ( 0 >= EntrySize ) return( false );
    
    return( true );
};

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

Trace_Lock::Trace_Lock( tsp00_TaskId TaskId )
:
m_Owner( TaskId ),
m_RegionId( g08trace )
{
    vbegexcl( m_Owner, m_RegionId );
};

/*---------------------------------------------------------------------------*/

Trace_Lock::~Trace_Lock()
{
    vendexcl( m_Owner, m_RegionId );
};

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/SAPDB/Trace/Trace_Manager.cpp <==
/****************************************************************************

  module      : Trace_Manager.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : 

  last changed: 2000-11-27  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "heo51.h"    // RTE   : VgetUKTId
#include "heo55k.h"   // RTE   : Vbegexcl/Vendexcl 
#include "heo56.h"    // RTE   : Vsuspend
#include "hgg01.h"    // PASCAL: Configuration
#include "hgg01_3.h"  // PASCAL: Configuration
#include "hgg08.h"    // PASCAL: RegionIds

#include "RunTime/MemoryManagement/RTEMem_Allocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "Trace/Trace_Manager.hpp"
#include "Trace/Trace_Writer.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

Trace_Manager*  Trace_Manager::m_Instance = NULL;

/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

class Trace_Lock
{
public:
    
    Trace_Lock( tsp00_TaskId TaskId );

    ~Trace_Lock();

private:

    tsp00_TaskId    m_Owner;
    tsp00_RegionId  m_RegionId;
};

/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/


/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/* Singleton interface                                                       */
/*---------------------------------------------------------------------------*/

bool
Trace_Manager::CreateInstance( tsp00_TaskId TaskId )
{
    if ( NULL == m_Instance )
    {
        SAPDB_Int4  NumUKTs = g01NumberOfUKTs();

        m_Instance = new( RTEMem_Allocator::Instance() ) Trace_Manager( NumUKTs );

        if (NULL == m_Instance ) return( false );

        if (! m_Instance->Initialize( TaskId )) return( false );
    }
    return( true );
};

/*---------------------------------------------------------------------------*/

Trace_Manager::Trace_Manager( SAPDB_Int4    NumUKTs )
:
m_TraceWriterRegistered( false ),
m_NumUKTs( NumUKTs ),
m_TraceWriterRequested ( 0 ),
m_SeqNoGenerator( TRACE_FIRST_SEQNO )
{
};

/*---------------------------------------------------------------------------*/

bool
Trace_Manager::Initialize( tsp00_TaskId TaskId )
{

    
    m_DevSpace.Initialize( m_DumpFile );
    
    if (! m_DevSpace.Open( TaskId )) return( false );

    if (! m_DevSpace.WriteInfoPage( TaskId )) return( false );

    return( m_SlotPool.Initialize( m_NumUKTs ) );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::Flush( tsp00_TaskId TaskId )
{
    Trace_SlotPoolIterator  SlotIter( m_SlotPool );
    
    if ( ! m_TraceWriterRegistered ) 
        return;
    
    // Flush memory trace into knltrace file
    for( SlotIter.Begin(); !SlotIter.End(); ++SlotIter )
    {
        ( *SlotIter ).Flush( TaskId, m_DevSpace);
    }
    {
        Trace_Lock  Lock( TaskId );
        
        // Resume user task(s) waiting for trace flush
        if( ! m_UserWaitFlushQueue.IsEmpty( TaskId ) ) // PTS 1112933 TS 2001-12-04
        {
            m_UserWaitFlushQueue.PopAllTaskIdsAndResume( TaskId ); 
        }
        // One requested trace flush is executed
        --m_TraceWriterRequested; // PTS 1112933 TS 2001-12-04
    }
};

/*---------------------------------------------------------------------------*/

tsp00_VFilename 
Trace_Manager::GetDumpFile()
{
    return( m_DumpFile );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::Insert(
                      tgg00_TransContext    &Trans,
                      Trace_IBaseEntry      &Entry )
{
    if (! WriteEntryPossible( Trans, Entry.GetSize() )) 
        return;

    // This method is not able to materialize trace entries greater than 132 bytes
    if ( Entry.MaximumSize() < Entry.GetSize() ) 
        return;
    
    // Get the RTE internal number of the UKT in which the current task is running.
    SAPDB_Int4  OwnUKTId = 0;
    
    vgetuktid( OwnUKTId );
    
    m_SlotPool.Get( OwnUKTId ).Insert( Trans, OwnUKTId, Entry, m_SeqNoGenerator );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::InsertLarge( 
                                tgg00_TransContext  &Trans,
                                Trace_ILargeEntry   &Entry )
{
    if (! WriteEntryPossible( Trans, Entry.GetSize() )) return;

    // Get the RTE internal number of the UKT in which the current task is running.
    SAPDB_Int4  OwnUKTId   = 0;
    
    vgetuktid( OwnUKTId );
    
    m_SlotPool.Get( OwnUKTId ).InsertLarge( Trans, OwnUKTId, Entry, m_SeqNoGenerator );
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::PrepareAndWaitForFlush( 
                                      tsp00_TaskId TaskId,
                                      SAPDB_Bool   bReInitialization )
{
    Trace_SlotPoolIterator  SlotIter( m_SlotPool );
    
    if ( ! m_TraceWriterRegistered ) 
        return;
    
    if (true == bReInitialization)
    {
        // Reset the sequence number generater for the trace entries
        m_SeqNoGenerator.SetValue( TRACE_FIRST_SEQNO );
        
        for( SlotIter.Begin(); !SlotIter.End(); ++SlotIter )
        {
            ( *SlotIter ).ReInitialization();
        }
    };
    {
        Trace_Lock  Lock( TaskId );
        
        // Resume the trace writer to flush UKT specific trace slots
        if ( 0 == m_TraceWriterRequested ) // PTS 1112933 TS 2001-12-04
        {
            Trace_Writer::GetInstance().Resume();
            ++m_TraceWriterRequested;
        }
        
        // Store user taskid to enable wake up after the
        // trace writer has flushed the complete trace.
        m_UserWaitFlushQueue.PushTaskId( TaskId ); // PTS 1112933 TS 2001-12-04
    }
    vsuspend (TaskId, cgg_suspend_lvl_0, 51);
};

/*---------------------------------------------------------------------------*/

void 
Trace_Manager::TraceWriterRegistration( tsp00_TaskId TaskId )
{
    Trace_Lock Lock( TaskId );
    
    m_TraceWriterRegistered = true;
};

/*---------------------------------------------------------------------------*/

bool
Trace_Manager::WriteEntryPossible( 
                                  tgg00_TransContext   &Trans,
                                  SAPDB_Int4           EntrySize )
{
    if ( ! m_TraceWriterRegistered ) 
        return( false );
    
    // if session specific trace is enabled than only the specified session 
    // has the right to write into the kernel trace.
    if ( 
        (! g01vtrace.vtrSession_gg00.gg90IsNil() ) 
        && 
        ( g01vtrace.vtrSession_gg00 != Trans.trSessionId_gg00 ) 
        &&
        (! Trans.trSessionId_gg00.gg90IsNil() )
        )
        return( false );
    
    if  ( 0 >= EntrySize ) 
        return( false );
    
    return( true );
};

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

Trace_Lock::Trace_Lock( tsp00_TaskId TaskId )
:
m_Owner( TaskId ),
m_RegionId( g08trace )
{
    vbegexcl( m_Owner, m_RegionId );
};

/*---------------------------------------------------------------------------*/

Trace_Lock::~Trace_Lock()
{
    vendexcl( m_Owner, m_RegionId );
};

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
